from credmgr import CredentialManager

credential_manager = CredentialManager()

bot = credential_manager.bot("bot_name")
reddit = bot.reddit_app.reddit("authorized_redditor")

subreddit = reddit.subreddit("dankmemes")
stream = subreddit.stream.comments()

while True:
    for comment in stream:
        if "mods gay" in comment.body and not comment.saved:
            comment.reply("no u")
            comment.save()
